package com.tanpu.community.api.beans.req.page;

import com.github.pagehelper.PageHelper;
import lombok.Builder;
import lombok.Data;

import java.util.List;

@Data
@Builder
public class Page<T> {

    private Integer pageNum;
    private Integer pageSize;
    private Long totalSize;
    private Integer totalPages;
    private List<T> content;

    public Page(Pageable pageable, List<T> content) {
        this.pageNum = pageable.getPageNumber();
        this.pageSize = content.size();
        this.content = content;
    }

    public Page(Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page page = PageHelper.startPage(pageNum, pageSize);

        this.pageNum = page.getPageNum();
        this.pageSize = page.getPageSize();
        this.totalSize = page.getTotal();
        this.content = page.getResult();
    }

    public Page(Pageable pageable, Long totalSize) {
        this.pageNum = pageable.getPageNumber();
        this.pageSize = pageable.getPageSize();
        this.totalSize = totalSize;
        if (totalSize == 0) {
            this.totalPages = 1;
        } else {
            this.totalPages = (int) (totalSize%pageable.getPageSize() == 0 ? totalSize/pageable.getPageSize() : totalSize/pageable.getPageSize() + 1);
        }
    }

    public Page(com.github.pagehelper.Page page) {
        this.pageNum = page.getPageNum();
        this.pageSize = page.getPageSize();
        this.totalSize = page.getTotal();
        this.totalPages = page.getPages();
        this.content = page.getResult();
    }

    public Page(com.github.pagehelper.Page page, List<T> content) {
        this.pageNum = page.getPageNum();
        this.pageSize = page.getPageSize();
        this.totalSize = page.getTotal();
        this.totalPages = page.getPages();
        this.content = content;
    }

    public Page(Pageable pageable, Long totalSize, List<T> content) {
        this.pageNum = pageable.getPageNumber();
        this.pageSize = pageable.getPageSize();
        this.totalSize = totalSize;
        this.content = content;
    }

    public Page(org.springframework.data.domain.Page page) {
        this.pageNum = page.getPageable().getPageNumber();
        this.pageSize = page.getPageable().getPageSize();
        this.totalSize = (long) page.getSize();
        this.totalPages = page.getTotalPages();
        this.content = page.getContent();
    }

    public Page(org.springframework.data.domain.Page page, List<T> content) {
        this.pageNum = page.getPageable().getPageNumber() + 1;
        this.pageSize = page.getPageable().getPageSize();
        this.totalSize = page.getTotalElements();
        this.totalPages = page.getTotalPages();
        this.content = content;
    }

    public Page(Integer pageNum, Integer pageSize, Long totalSize) {
        this.pageNum = pageNum;
        this.pageSize = pageSize;
        this.totalSize = totalSize;
        if (totalSize == 0) {
            this.totalPages = 1;
        } else {
            this.totalPages = (int) (totalSize%pageSize == 0 ? totalSize/pageSize : totalSize/pageSize + 1);
        }
    }

    public Page(Integer pageNum, Integer pageSize, Long totalSize, Integer totalPages, List<T> content) {
        this.pageNum = pageNum;
        this.pageSize = pageSize;
        this.totalSize = totalSize;
        this.totalPages = totalPages;
        this.content = content;
    }

    public Page() {
    }

    @Override
    public String toString() {
        return "Page{" +
                "pageNum=" + pageNum +
                ", pageSize=" + pageSize +
                ", content=" + "[...]" +
                '}';
    }
}
