package com.tanpu.community.feign.product;

import com.tanpu.common.api.CommonResp;
import com.tanpu.community.api.beans.vo.feign.fund.FundCompanySimpleVO;
import com.tanpu.community.api.beans.vo.feign.product.ProductInfoVO;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;

@Slf4j
@Component
public class FeignbackForFund implements FallbackFactory<FeignForFund> {

    @Override
    public FeignForFund create(Throwable throwable) {
        return new FeignForFund() {
            @Override
            public CommonResp<List<ProductInfoVO>> getProductList(List<String> ids) {
                log.error("FeignbackForFund.getProductList");
                return CommonResp.failed("调用私募详情失败");
            }

            @Override
            public CommonResp<List<ProductInfoVO>> getPrivateFundList(List<String> ids) {
                log.error("FeignbackForFund.getPrivateFundList");
                return CommonResp.failed("调用ifa导入基金失败");
            }
            @Override
            public CommonResp<List<ProductInfoVO>> getProductListNotNetInfo(List<String> ids) {
                log.error("FeignbackForFund.getProductListNotNetInfo", throwable);
                return CommonResp.failed("查询无净值产品");
            }
            @Override
            public CommonResp<FundCompanySimpleVO> getFundCompanyInfoSimple(String id) {
                log.info("FeignForFund.getFundCompanyInfoSimple 调用失败 id:{}", id);
                log.error("FeignForFund.getFundCompanyInfoSimple 调用失败", throwable);
                return CommonResp.error();
            }

            @Override
            public CommonResp<FundCompanySimpleVO> getCompanyFundCount(String id) {
                log.info("FeignForFund.getCompanyFundCount 调用失败 id:{}", id);
                log.error("FeignForFund.getCompanyFundCount 调用失败", throwable);
                return CommonResp.error();
            }
        };
    }
}
