package com.tanpu.community.manager;

import com.tanpu.community.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

@Slf4j
@Service
@Configuration
public class ConJobManager {

    @Autowired
    private VisitLogService visitLogService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private RankService rankService;

    @Autowired
    private RankLogService rankLogService;

    @Autowired
    private RecommendService recommendService;

    /**
     * 定时统计 话题 访问数据，并刷到redis
     */
    @Scheduled(cron = "*/10 * * * * ?")
    public void topicVisitorStats() {
        String topicId = "123";
        Integer detailVisitTimes = visitLogService.queryTopicDetailVisit(topicId);
        redisService.set("topicVisitorStats", detailVisitTimes);
    }

    /**
     * 定时统计主题、话题排行
     */
    @Scheduled(cron = "*/30 * * * * ?")
    public void themeRank() {
        rankService.rankThemes();
        rankService.rankTopics();
    }

    /**
     * 定时统计主题、话题排行
     */
    @Scheduled(cron = "*/5 * * * * ?")
    public void getThemeNewest() {
        recommendService.refreshNewestThemes();
    }

    /**
     * 定时把rank_log的日志拿出来，清理数据库
     */
    @Scheduled(cron = "0 0 0 ? * 1")
    public void clearRankLog() {
        rankLogService.clearRankLog();
    }
}
